<?php
/**
 * W3 XooguGeotargetAdmin module
 */
if (!defined('W3TC')) {
    die();
}
class W3_XooguGeotargetAdmin {
    
    protected $aCountries;
    protected $sDefaultCountry;
	protected $aCountryList;
	protected $aErrors;
	protected $aErrorMsgs;
    
    function run() {
        load_plugin_textdomain('xoogu-geotarget-w3tc', false, dirname(__FILE__).'/XooguGeoTarget/lang/');
        add_filter('w3tc_extensions', array($this, 'extension'), 10, 2);
		add_action('w3tc_extensions_admin_init', array($this, 'admin_init'));
        add_action('w3tc_extensions_page-xoogu-geotarget-w3tc', array($this, 'admin_head_page'));
		//W3TC doesn't seem to actually apply the sanitization filters when saving the extension settings, so have to do it manually
        add_filter("w3tc_save_extension_settings-xoogu-geotarget-w3tc", array($this, 'save_extension_settings'), null, 2);
        //Add activation and deactivation handlers to modify advanced-cache.php
        add_action('w3tc_activate_extension-xoogu-geotarget-w3tc', array($this, 'activate'));
        add_action('w3tc_deactivate_extension-xoogu-geotarget-w3tc', array($this, 'deactivate'));
    }

    
    /**
	 *Initializes the settings for the admin page
	 */
	public function admin_init()
    {
		//get the list of all country codes and names
        include(dirname(__FILE__).'/XooguGeotarget/countrylist.php');
		$this->aCountryList = $aCountryList;
		
		//potential error messages
		$this->aErrorMsgs = array(
			'noCountries' => __('You must choose at least two countries that will see different content', 'xoogu-geotarget-w3tc'),
			'countryCodeInvalid' => __('%s is not a valid country code', 'xoogu-geotarget-w3tc'),
			'noDefaultCountry' => __('You must choose a default country', 'xoogu-geotarget-w3tc')
		);
		
		//get the extension settings
        $this->aCountries = w3tc_get_extension_config('xoogu-geotarget-w3tc', 'countries', null, array_keys($this->aCountryList));
        $this->sDefaultCountry = w3tc_get_extension_config('xoogu-geotarget-w3tc', 'default-country', null, 'US');
		
		//If there was an error message, then remove W3TC's automatically added note that the settings were updated (quite hacky)
		$aErrMsgs = get_transient( 'anh_notices' ); //empty can only be used to check vars, not function return vals, so have to put val in a var to check it
		if (!empty($aErrMsgs)) {
			add_filter('w3tc_notes', array($this, 'removeSavedNoteOnError'));
		}
	}
	
	
	/**
	 *Calls the functions for showing the settings form on the extension admin page
	 */
	public function admin_head_page()
	{
		wp_enqueue_style('xoogu_geotext_admin', plugins_url( '/XooguGeotarget/admin.css' , __FILE__ ));
        
        w3tc_add_settings_section(
            'xgt_settings', // Unique identifier for the settings section
            __('Settings','xoogu-geotarget-w3tc'), // Section title
            '__return_false', // Section callback (we don't want anything)
            'xoogu-geotarget-w3tc' // extension id, used to uniquely identify the extension;
        );
        w3tc_register_setting('xoogu-geotarget-w3tc', 'countries', array($this, 'sanitize_countries'));
        w3tc_add_settings_field("countries", __('Countries that pages should be cached separately for','xoogu-geotarget-w3tc'),
            array($this, 'printSettingsCountries'), 'xoogu-geotarget-w3tc', 'xgt_settings',
            array('label_for'=>"extensions_settings_xoogu-geotarget-w3tc_countries"));
        w3tc_register_setting('xoogu-geotarget-w3tc', 'default-country', array($this, 'sanitize_defaultCountry'));
        w3tc_add_settings_field("default-country", __('Default country','xoogu-geotarget-w3tc'),
            array($this, 'printSettingsDefaultCountry'), 'xoogu-geotarget-w3tc', 'xgt_settings',
            array('label_for'=>"extensions_settings_xoogu-geotarget-w3tc_default-country"));
    }
	
	
	/**
	 *Validates a new countries setting. Returns the new setting as an array of country codes if it was valid, otherwise returns the old setting
	 *@param string $sValue List of comma separated country codes to save as the value
	 *@return array Array of country codes to save as the value
	 */
	public function sanitize_countries($sValue)
	{
		//ensure country codes are uppercase
		$sValue = strtoupper(trim($sValue));
		//check the text field wasn't empty
		if (empty($sValue)) {
			$this->aErrors['noCountries'] = $this->aErrorMsgs['noCountries'];
			return $this->aCountries;
		} else {
			$aValue = explode(',', $sValue);
			//holds any invalid country codes
			$aCountryCodeInvalid = array();
			//holds our list of country codes to save
			$aValidCountries = array();
			//loop through the country codes and check each one
			foreach ($aValue as $sCountryCode) {
				$sCountryCode = trim($sCountryCode);
				if (empty($sCountryCode)) {
					continue;
				}
				//check country code is valid
				if (empty($this->aCountryList[$sCountryCode])) {
					$aCountryCodeInvalid[] = sprintf($this->aErrorMsgs['countryCodeInvalid'], $sCountryCode);
				} else {
					//use $sCountryCode as key to ensure no country code added more than once
					$aValidCountries[$sCountryCode] = $sCountryCode;
				}
			}
			//check all country codes entered were valid
			if (!empty($aCountryCodeInvalid)) {
				$this->aErrors['countryCodeInvalid'] = $aCountryCodeInvalid;
				return $this->aCountries;
			}
			//check we have at least two country codes to save as the setting value
			if (count($aValidCountries) < 2) {
				$this->aErrors['noCountries'] = $this->aErrorMsgs['noCountries'];
				return $this->aCountries;
			}
			//If everything was OK (no errors at all), then return our list of country codes to save as the setting value
			return $aValidCountries;
		}
	}
	
	
	/**
	 *Sanitizes a new default country setting. Returns the new setting if valid, otherwise returns the old setting.
	 *@param string $sValue The new default country setting to save (2 digit country code)
	 *@return string The default country setting to save
	 */
	public function sanitize_defaultCountry($sValue)
	{
		//make sure country code is uppercase
		$sValue = strtoupper(trim($sValue));
		//check a value was actually provided
		if (empty($sValue)) {
			$this->aErrors['noDefaultCountry'] = $this->aErrorMsgs['noDefaultCountry'];
			return $this->sDefaultCountry;
		//check it's a valid country code
		} else if (empty($this->aCountryList[$sValue])) {
			$this->aErrors['defaultCountryCodeInvalid'] = sprintf($this->aErrorMsgs['countryCodeInvalid'], $sValue);
			return $this->sDefaultCountry;
		} else {
			return $sValue;
		}
	}
    
	
    /**
     *Prints the input for adding country codes and a drop down country select helper
     *@param string $sSetting The setting id
     *@param array $aArgs Any additional arguments supplied
     */
    public function printSettingsCountries($sSetting, $aArgs)
    {
        list($sName, $sId) = w3tc_get_name_and_id('xoogu-geotarget-w3tc', $sSetting);
        //get the current value
        $sValue = '';
        if (isset($_POST[$sName])) {
            $sValue = $_POST[$sName];
        } elseif (!empty($this->aCountries)) {
            $sValue = implode(',', $this->aCountries);
        }
		
        //text input
        echo '<input type="text" name="'.$sName.'" id="'.$sId.'" value="'.$sValue.'" autocomplete="off" />';
        //drop down helper
        echo '<select id="'.$sId.'-helper" autocomplete="off">';
        echo '<option value="">Add a country</option>';
        echo '<option value="all">Add all countries</option>';
        foreach ($this->aCountryList as $sCountryCode => $sCountryName) {
            echo '<option value="'.$sCountryCode.'">'.$sCountryName.'</option>';
        }
        echo '</select>';
        
        wp_enqueue_script('xoogu-geotarget-w3tc-admin', plugins_url('XooguGeotarget/admin.js', __FILE__), null, null, true);
        wp_localize_script( 'xoogu-geotarget-w3tc-admin', 'xgtL10n', $this->aErrorMsgs );
    }
    
	
    /**
     *Prints the input for adding country codes and a drop down country select helper
     *@param string $sSetting The setting id
     *@param array $aArgs Any additional arguments supplied
     */
    public function printSettingsDefaultCountry($sSetting, $aArgs)
    {
        list($sName, $sId) = w3tc_get_name_and_id('xoogu-geotarget-w3tc', $sSetting);
        //get the current value
        $sValue = '';
        if (isset($_POST[$sName])) {
            $sValue = $_POST[$sName];
        } elseif (!empty($this->sDefaultCountry)) {
            $sValue = $this->sDefaultCountry;
        }
        //text input
        echo '<input type="text" name="'.$sName.'" id="'.$sId.'" value="'.$sValue.'" />';
    }
	
	
	/**
	 *Removes the 'Plugin configuration successfully updated' message from the list of notes to be printed
	 *(Call when there is an error saving the plugin settings)
	 *@param array $notes Array of notes to print
	 *@return array Array of notes to print
	 */
	public function removeSavedNoteOnError($notes)
	{
		//Notes is not an associative array, so we need to try and check each value to see if it is the updated message
		//TotalCacheAdmin.php ln 508
		if ( ($key = array_search(__('Plugin configuration successfully updated.', 'w3-total-cache'), $notes)) !== false) {
			unset($notes[$key]);
		}
		return $notes;
	}
	
	
	/**
	 *W3TC doesn't seem to actually apply the settings sanitization specified when registering the settings, so do it manually by hooking into the w3tc_save_extension_settings-xoogu-geotarget-w3tc filter
	 *@param W3_Config $extension_settings the new settings to be saved
	 *@param W3_Config $extension_settings the old settings
	 *@return W3_Config The sanitized new extension settings to save
	 */
	public function save_extension_settings($extension_settings, $old_extension_settings)
	{
		$extension_settings['default-country'] = $this->sanitize_defaultCountry($extension_settings['default-country']);
		$extension_settings['countries'] = $this->sanitize_countries($extension_settings['countries']);
		if (!empty($this->aErrors)) {
			add_notice( $this->printErrors($this->aErrors), 'error' );
		}
		return $extension_settings;
	}
	
	
	/**
	 *Saves the default settings for the extension
	 *(Call from activate)
	 */
	public function add_extension_settings()
	{
		$oConfig = w3_instance('W3_Config');
		$aExtensionsData = $oConfig->get_array('extensions.settings', array());
		//check the setting doesn't already exist
		if (empty($aExtensionsData['xoogu-geotarget-w3tc'])) {
			//add the settings
			$aExtensionsData['xoogu-geotarget-w3tc'] = array(
				'default-country' => $this->sDefaultCountry,
				'countries' => $this->aCountries
			);
			$oConfig->set('extensions.settings', $aExtensionsData);
			$oConfig->save();
		}
	}
	
	
	/**
	 *Deletes the extension settings
	 *(Call manually to clear the settings when testing)
	 */
	public function delete_extension_settings()
	{
		$oConfig = w3_instance('W3_Config');
		$aExtensionsData = $oConfig->get_array('extensions.settings', array());
		//check the setting isn't already empty
		if (!empty($aExtensionsData['xoogu-geotarget-w3tc'])) {
			//remove the setting
			unset($aExtensionsData['xoogu-geotarget-w3tc']);
			$oConfig->set('extensions.settings', $aExtensionsData);
			$oConfig->save();
		}
	}
	
	
	/**
	 *Creates an HTML unordered list filled with error messages
	 *@param array $aErrors An associative array with error messages as the values, or further arrays containing error messages as the values
	 *@return string HTML unordered list containing all the error messages
	 */
	public function printErrors($aErrors)
	{
		$sErrStr = '<ul>';
		foreach ($aErrors as $mError) {
			if (is_array($mError)) {
				$sErrStr .= '<li>'.$this->printErrors($mError).'</li>';
			} else {
				$sErrStr .= '<li>'.$mError.'</li>';
			}
		}
		return $sErrStr.'</ul>';
	}
	

    /**
     * Register the extension with W3TC
     * @param array $aExtensions The extensions available
     * @param W3_Config $oConfig
     * @return array Extensions available
     */
    function extension($aExtensions, $oConfig) {
        $aEnabled = $this->isEnabled();
        $aExtensions['xoogu-geotarget-w3tc'] = array (
            'name' => __('Geo targeted caching', 'xoogu-geotarget-w3tc'),
            'author' => 'xoogu.com',
            'description' => __('Cache pages on a per country basis', 'xoogu-geotarget-w3tc'),
            'author uri' => 'http://www.xoogu.com/',
            'extension uri' => 'http://www.xoogu.com/geo-targeted-caching-extension-for-w3-total-cache/',
            'extension id' => 'xoogu-geotarget-w3tc',
            'version' => '2015-07-28',
            'enabled' => $aEnabled['bEnabled'],
            'requirements' => implode('<br />',$aEnabled['aMsg']),
            'path' => 'w3-total-cache/extensions/XooguGeotarget/nada.php' //This is an empty file since we load the actual extension much earlier via advanced_cache.php
        );

        return $aExtensions;
    }
    
    
    /**
     *Detects whether the extension should be available to activate or not
     *@return array Array with two items:
     * 'bEnabled' => bool Whether the extension should be available to activate or not
     * 'aMsg' => array List of reasons why the extension is not available to activate
     */
    protected function isEnabled()
    {
        $bEnabled = true;
        $aMsg = array();
        $oConfig = w3_instance('W3_Config');
        if (!$oConfig->get_boolean('pgcache.enabled')) {
            $aMsg[] = __('Page cache is not enabled.', 'xoogu-geotarget-w3tc');
            $bEnabled = false;
        } elseif ($oConfig->get_string('pgcache.engine') !== 'file_generic') {
            $aMsg[] = __('Page cache method must be \'Disk: Enhanced\'.', 'xoogu-geotarget-w3tc');
            $bEnabled = false;
        }
        if (!$this->detectGeoMethod()) {
            $aMsg[] =__('Requires your web server has GeoIP functionality available or the PHP GeoIP extension is installed. It does not appear that either are available.', 'xoogu-geotarget-w3tc');
            $bEnabled = false;
        }
        return array('bEnabled' => $bEnabled, 'aMsg' => $aMsg);
    }
    
    
    /**
     *Detects the geomethod available (server or php extension)
     *@return string The geomethod or an empty string if none found
     */
    public function detectGeoMethod()
    {
        if (isset($_SERVER['GEOIP_COUNTRY_CODE'])) {
            $sGeoMethod = 'server';
        } elseif (function_exists('geoip_country_code_by_name')) {
            $sGeoMethod = 'php';
        } else {
            $sGeoMethod = '';
        }
        return $sGeoMethod;
    }
    
    
    /**
     * W3TC only loads the extension after checking if a cached file exists and after
     * purging cached files. Since we need to hook into both of these actions, this
     * means we have to edit some files to manually load our extension before the
     * actions we need to hook into.
     * advanced_cache.php is edited to load the extension for modifying the page key
     * w3-total-cache.php (The main W3TC plugin file) is edited to load our cache
     * purge
     */
    public function activate()
    {
        //We edit the copy of advanced_cache.php stored in the W3TC plugin directory
        //W3TC automatically monitors whether the WP advanced_cache.php matches the
        //W3TC copy, and if it doesn't, then it overwrite's WP's version with its own.
        //So by modifying the W3TC copy, W3TC will automatically update the WP copy.
        //(If we do it the other way round, W3TC will overwrite any changes we make
        //to the WP copy)
        
        //need to include file to make w3_wp_write_to_file() available
        w3_require_once(W3TC_INC_DIR . '/functions/activation.php');
        $this->add_to_file(W3TC_INSTALL_FILE_ADVANCED_CACHE, '$w3_pgcache = w3_instance(\'W3_PgCache\');', $this->advanced_cache_addon());
        $this->add_to_file(WP_PLUGIN_DIR.'/'.W3TC_FILE, '$root = w3_instance(\'W3_Root\');', $this->w3tc_file_addon());
        $this->add_extension_settings();
    }
    
    
    /**
     *Modify advanced-cache.php upon deactivation
     */
    public function deactivate()
    {        
        //need to include file to make w3_wp_write_to_file() available
        w3_require_once(W3TC_INC_DIR . '/functions/activation.php');
        $this->remove_from_file(W3TC_INSTALL_FILE_ADVANCED_CACHE, $this->advanced_cache_addon());
        $this->remove_from_file(WP_PLUGIN_DIR.'/'.W3TC_FILE, $this->w3tc_file_addon());
		//$this->delete_extension_settings();
    }
    
    
    /**
     *Finds specified text in a file, inserts the new text before this point, then saves the file
     *@param string $sFilename Path of the file to edit
     *@param string $sInsertBefore The text to find in the file where the new text will be inserted
     *@param string $sTextToAdd The text to insert
     */
    protected function add_to_file($sFilename, $sInsertBefore, $sTextToAdd)
    {
        //Based on W3_PgCacheAdminEnvironment::wp_config_add_directive()
        $config_data = @file_get_contents($sFilename);
        if (!$config_data) {
            return;
        }
        
        //check our line doesn't already exist
        if ($this->check_XooguGeotarget_mentioned($config_data)) {
            return;
        }
        
        //Add our line
        $new_config_data = str_replace(
            $sInsertBefore, 
            $sTextToAdd.$sInsertBefore, 
            $config_data,
            $iCount);
        
        if ($iCount !== 1) {
            add_notice( sprintf(__("Couldn't find %s in file %s to add extra code, please manually check file and add line of code: %s", 'xoogu-geotarget-w3tc'), "<code>$sInsertBefore</code>", $sFilename, "<code>$sTextToAdd</code>"), 'error' );
            return;
        }
        
        
        try {
            w3_wp_write_to_file($sFilename, $new_config_data);
        } catch (FilesystemOperationException $ex) {
            throw new FilesystemModifyException(
                $ex->getMessage(), $ex->credentials_form(),
                sprintf(__('Edit file %s and before %s add line:', 'xoogu-geotarget-w3tc'), ' <strong>' . $sFilename . 
                '</strong> ', "<code>$sInsertBefore</code>"), $sFilename, $sTextToAdd
            );
        }
    }
    
    
    /**
     *Removes specified text from the given file
     *@param string $sFilename Path of the file to edit
     *@param string $sTextToRemove The text to remove
     */
    protected function remove_from_file($sFilename, $sTextToRemove)
    {
        //Based on W3_PgCacheAdminEnvironment::wp_config_add_directive()
        $config_data = @file_get_contents($sFilename);
        if (!$config_data) {
            return;
        }
        
        //check our line doesn't already exist
        if (!$this->check_XooguGeotarget_mentioned($config_data)) {
            return;
        }
        
        //Remove our line
        $new_config_data = str_replace(
            $sTextToRemove, 
            '', 
            $config_data,
            $iCount);
        if ($iCount !== 1) {
            add_notice( sprintf(__("Couldn't find %s in file %s to remove it, please manually check file and remove line of code that mentions %s", 'xoogu-geotarget-w3tc'), "<code>$sTextToRemove</code>", $sFilename, 'XooguGeotarget'), 'error' );
            return;
        }
        
        try {
            w3_wp_write_to_file($sFilename, $new_config_data);
        } catch (FilesystemOperationException $ex) {
            throw new FilesystemModifyException(
                $ex->getMessage(), $ex->credentials_form(),
                sprintf(__('Edit file %s and remove line:', 'xoogu-geotarget-w3tc'), ' <strong>' . $sFilename . 
                '</strong> '), $sFilename, $sTextToRemove
            );
        }
    }
    
    
    /**
     *Checks our config line has been added to the given text
     *@param string $config_data (optional) the config text to check for our config line (defaults to the contents of the advanced_cache.php file)
     *@return bool Whether the config line exists
     */
    protected function check_XooguGeotarget_mentioned($config_data)
    {
        if (!is_string($config_data)) {
            $config_data = @file_get_contents(W3TC_INSTALL_FILE_ADVANCED_CACHE);
        }
        if (!$config_data) {
            return false;
        }
        
        //check our line doesn't already exist
        if (strrpos($config_data,'XooguGeotarget')) {
            return true;
        }
        
        return false;
    }
	
    
	/**
	 *Line of text to add to the advanced_cache.php file, to ensure the extension is loaded early enough to hook into W3TC
	 *@return string The text to add to the file
	 */
    protected function advanced_cache_addon()
    {
        return "include(W3TC_DIR . '/extensions/XooguGeotarget/XooguGeotarget.php');\n\t\t\t";
    }
    
	
    /**
	 *Line of text to add to the w3-total-cache.php file, to ensure the extension is loaded early enough to hook into W3TC
	 *(Needed to ensure that cache can be flushed properly)
	 *@return string The text to add to the file
	 */
    protected function w3tc_file_addon()
    {
        return "include_once(W3TC_DIR . '/extensions/XooguGeotarget/flushcache.php');\n\t";
    }
}

$ext = new W3_XooguGeotargetAdmin();
$ext->run();

//After activating / deactivating the extension W3TC does a redirect. So to get any error messages to show up
//we'll use this helper class
include_once(dirname(__FILE__).'/XooguGeotarget/admin-notice-helper/admin-notice-helper.php');